import os
import maya.cmds as cmds
import hmScripts as root
import hmScripts.hm_global_variables as global_vars
import hmScripts.hm_global_variables as nc
import hmScripts.UI.HmMenu_OperatorEdit as edit_menu
from hmScripts.hm_global_variables import language


mayaVersion = int( str(cmds.about(api=True))[0:4] )
rootPath = root.__path__ [0]

# Get the icons path starting from the script roots
abs = os.path.abspath(rootPath)
parent_path = os.path.dirname( os.path.dirname(abs) )
icons_path = os.path.join(parent_path, 'icons')

tooltips = language.values['stack_win']['tooltips']
labels = language.values['stack_win']
op_labels = language.values['new_operation_win']

class OperatorsStack():
    """
    This class implements the UI elements for the Hard Mesh Operators Stack.
    In can be displayed any window but needs to be connected to an HmNetworkHandler in order to work.
    """
    # === Attributes

    # This is a feature that can be triggered from the main UI
    AUTO_LOCK_UNLOCK_RESULT = 1

    # Visible icon
    __visible_icons = ['', os.path.join(icons_path, 'hm2_visible.png')]


    # Operation icons
    __iconA = os.path.join(icons_path, 'hm2_opUnion.png')
    __iconB = os.path.join(icons_path, 'hm2_opDiffAB.png')
    __iconC = os.path.join(icons_path, 'hm2_opDiffBA.png')
    __iconD = os.path.join(icons_path, 'hm2_opInter.png')
    __iconE = os.path.join(icons_path, 'hm2_opPanAB.png')
    __iconF = os.path.join(icons_path, 'hm2_opPanBA.png')

    # Status icons
    __icon_status_start_1 = os.path.join(icons_path, 'hm2_opStatus_start_1.png' )
    __icon_status_start_0 = os.path.join(icons_path, 'hm2_opStatus_start_0.png' )
    __icon_status_mid_1 = os.path.join(icons_path, 'hm2_opStatus_mid_1.png' )
    __icon_status_mid_0 = os.path.join(icons_path, 'hm2_opStatus_mid_0.png' )
    __icon_status_end_0 = os.path.join(icons_path, 'hm2_opStatus_end_0.png' )
    __icon_status_end_1 = os.path.join(icons_path, 'hm2_opStatus_end_1.png' )

    __icon_status_any_0 = os.path.join(icons_path, 'hm2_opStatus_any_0.png' )


    # Icons arrays
    __OPERATION_LABELS = (op_labels['operation_a'],
                          op_labels['operation_b'],
                          op_labels['operation_c'],
                          op_labels['operation_d'],
                          op_labels['operation_e'],
                          op_labels['operation_f'])

    __OPERATION_ICONS = [__iconA, __iconB, __iconC, __iconD, __iconE, __iconF]
    __icon_status_start = [__icon_status_start_0, __icon_status_start_1]
    __icon_status_mid = [__icon_status_mid_0, __icon_status_mid_1]
    __icon_status_end = [__icon_status_end_0, __icon_status_end_1]

    # Tree view groups names
    __item_mesh_group_name = 'Input'
    __item_intersection_group_name = 'Curves'
    __item_result_name = 'Result - '

    __item_mesh_A_prefix = 'A - '
    __item_mesh_B_prefix = 'B - '

    # Tree items types
    ITEM_TYPE_NETWORK = 'network'
    ITEM_TYPE_INPUT_A = 'input_A'
    ITEM_TYPE_INPUT_B = 'input_B'
    ITEM_TYPE_RESULT = 'result'
    ITEM_TYPE_OTHER = 'other'
    __ITEM_TYPES = (ITEM_TYPE_NETWORK, ITEM_TYPE_INPUT_A, ITEM_TYPE_INPUT_B, ITEM_TYPE_RESULT, ITEM_TYPE_OTHER)

    # Print messages
    VERBOSE = 0

    # Add input mesh and intersections in the tree view item
    _ITEMS_INCLUDES_HIERARCHY = 0

    # Update or not
    UPDATE_ON_CLICK = True

    def __init__(self, hm_network_handler, attribute_object = None ):
        """
        Write here what the function does.

        Args:
            hm_network_handler: The instance oh HmNetworkHandler to use.

        Returns:
            OperatorsStack
        """

        # === The main layout of the UI
        self.main_parent = None

        # === Tree view control
        self.tree_view_control = None
        self.items_name_list = [] # this is already storing shit
        self.items_state = {}
        self.sub_item_selected = False

        # === Compute control
        self.cb_global_compute = None

        # === Visibility controls
        self.cb_base_meshes = None
        self.cb_result_mesh = None
        self.cb_curves = None
        self.btn_lock_unlock_res = None

        # === Handler Instance
        self.handler_instance = hm_network_handler


        # Initialize the edit menu
        self.edit_menu = edit_menu.OperatorMenu( self.handler_instance, self )


        # Optional attributes instance. Is optional in the case that we want the tree view
        # as a stand alone cntrol and we want to visualize attributes in another way
        self.attribute_object = attribute_object

    @property
    def ITEMS_INCLUDES_HIERARCHY(self):
        return self._ITEMS_INCLUDES_HIERARCHY

    @ITEMS_INCLUDES_HIERARCHY.setter
    def ITEMS_INCLUDES_HIERARCHY(self, value):
        self._ITEMS_INCLUDES_HIERARCHY = value
        if value:
            self.sub_item_selected = False

    # ---------------------------------------------------------------------------------------------------------------- #
    def create_ui_elements(self):
        """
        This method create the UI elements.
        Put it in whatever window or control layout to make it appear there.
        """

        self.main_parent = cmds.frameLayout( label = labels['title'],
                                             collapsable= False,
                                             collapse = False,
                                             enable = False )
        layout = cmds.formLayout()

        # ------------------------------------------- GLOBAL CONTROLS ------------------------------------------------ #
        # Add the specific form layout to organze the elements of this section
        globalsLayout = cmds.formLayout()

        # Add the elements into the form
        self.cb_global_compute = cmds.checkBox(label = labels['global_compute'],
                                               cc = self.handler_instance.set_compute_state,
                                               ann=tooltips['global_compute'] )

        text = cmds.text(label = '')
        plusBtn = cmds.iconTextButton(label = labels['increase'],
                                      ann=tooltips['increase'],
                                      style='iconAndTextHorizontal',
                                      width = 80,
                                      enableBackground =True,
                                      image1='hm2_increase_smoothing.png',
                                      c = lambda *_:self.change_smooth(1),)


        minusBtn = cmds.iconTextButton(label = labels['decrease'],
                                       ann = tooltips['decrease'],
                                       style='iconAndTextHorizontal',
                                       width = 80,
                                       enableBackground =True,
                                       image1='hm2_decrease_smoothing.png',
                                       c = lambda *_:self.change_smooth(0))


        # Align global controls
        cmds.formLayout(globalsLayout,e=True, attachForm=(self.cb_global_compute,'left', 2))
        cmds.formLayout(globalsLayout,e=True, attachForm=(minusBtn,'right', 2))

        cmds.formLayout(globalsLayout,e=True, attachControl=(plusBtn,'right', 2, minusBtn))
        cmds.formLayout(globalsLayout,e=True, attachControl=(text,'right', 2, plusBtn))

        # ------------------------------------------- OPERATORS LIST ------------------------------------------------- #
        # Set the parent to the main layout
        cmds.setParent( layout)

        # Add the specific form layout to organze the elements of this section
        operatorsLayout = cmds.formLayout()

        # Add the tree view and the popup menu
        self.tree_view_control = cmds.treeView( numberOfButtons = 3,
                                                abr = False,
                                                allowDragAndDrop = True,
                                                allowReparenting=False,
                                                allowMultiSelection = False)

        cmds.popupMenu()
        self.edit_menu.create_ui_elements()


        # === Callback commands
        cmds.treeView(self.tree_view_control,edit=True,pressCommand=[(1,self.__change_visibility_TreeCallBack),
                                                                    (3,self.__bypass_operator_TreeCallBack)])
        cmds.treeView(self.tree_view_control,edit=True,selectionChangedCommand=self.__selectTreeCallBack)
        cmds.treeView(self.tree_view_control,edit=True,dragAndDropCommand = self.__dragCallback)
        cmds.treeView(self.tree_view_control,edit=True,editLabelCommand = self.__edit_label_rename)
        cmds.treeView(self.tree_view_control,edit=True,expandCollapseCommand = self.__store_tree_state)
        # cmds.treeView(self.tree_view_control,edit=True,itemDblClickCommand = self.__item_double_click)

        meshColumnL = cmds.columnLayout()

        if global_vars.RELEASE >= 2.5:
            cmds.button(label = 'ADD', width = 40)
            cmds.button(label = 'REM', width = 40)
            cmds.separator(h=20)
            cmds.button(label = 'UP', width = 40)
            cmds.button(label = 'DOWN', width = 40)

        cmds.setParent( '..' )

        # Align global controls
        cmds.formLayout(operatorsLayout,e=True, attachForm=(self.tree_view_control,'top', 2))
        cmds.formLayout(operatorsLayout,e=True, attachForm=(self.tree_view_control,'left', 2))
        cmds.formLayout(operatorsLayout,e=True, attachForm=(self.tree_view_control,'bottom', 2))
        cmds.formLayout(operatorsLayout,e=True, attachForm=(meshColumnL,'right', 2))
        cmds.formLayout(operatorsLayout,e=True, attachControl=(self.tree_view_control,'right', 2, meshColumnL))

        # ---------------------------------------- VISIBILITY SECTION ------------------------------------------------ #
        cmds.setParent( layout)

        # Add the specific form layout to organize the elements of this section
        viewLayout = cmds.formLayout()

        self.global_local_coll =cmds.radioButtonGrp(labelArray2=(labels['all'], labels['current']),
                                                    numberOfRadioButtons=2,
                                                    onc = self.toggle_local_global_visibility,
                                                    vertical = True)


        # Add the elements into the form
        self.btn_wire_shad_base = cmds.shelfButton( image1="WireFrame.png",
                                                    c = self.toggle_wire_shade_input,
                                                    ann=tooltips['shading'])

        self.cb_base_meshes =      cmds.checkBox(label = labels['input'],
                                                 changeCommand =  self.__show_hide_in_meshes_and_update_lock_icon,
                                                 ann=tooltips['input'])

        self.cb_curves =           cmds.checkBox(label = labels['curves'],
                                                 changeCommand = self.handler_instance.set_curves_visibility,
                                                 ann=tooltips['curves'])

        self.cb_result_mesh =      cmds.checkBox(label = labels['result'],
                                                 changeCommand = self.handler_instance.set_out_mesh_visibility,
                                                 ann=tooltips['result']
                                                )

        self.btn_lock_unlock_res = cmds.shelfButton(image1="Lock_OFF.png",
                                                    c = self.toggle_lock_result,
                                                    ann=tooltips['lock'],
                                                    height = 5)

        col_l = cmds.columnLayout(adj =1, rowSpacing = 2 )
        wire_btn = cmds.iconTextButton(style='iconAndTextHorizontal',
                                       enableBackground =True,
                                       image1='WireFrameOnShaded.png',
                                       c=self.wireframeToggle,
                                       label = labels['wire_ico'],
                                       ann = tooltips['view_wireframe'],
                                       width = 65)


        btn = cmds.iconTextButton(style='iconAndTextHorizontal',
                                  enableBackground =True,
                                  image1='Shaded.png',
                                  c=self.hilightingToggle,
                                  label = labels['hilight_ico'],
                                  ann = tooltips['view_wireframe'],
                                  width = 65)


        cmds.setParent('..')

        # Organize layout
        cmds.formLayout(viewLayout, e=True, attachForm=(self.global_local_coll, 'left', 2))
        cmds.formLayout(viewLayout,e=True, attachForm=(col_l,'right', 2))

        cmds.formLayout(viewLayout, e=True, attachControl=(self.btn_wire_shad_base,'left', 2, self.global_local_coll))
        cmds.formLayout(viewLayout,e=True, attachForm= (self.btn_wire_shad_base, 'top', 2) )
        cmds.formLayout(viewLayout,e=True, attachForm= (self.btn_wire_shad_base, 'bottom', 2) )

        cmds.formLayout(viewLayout,e=True, attachControl=(self.cb_base_meshes,'left', 2, self.btn_wire_shad_base))
        cmds.formLayout(viewLayout,e=True, attachForm= (self.cb_base_meshes, 'top', 2) )
        cmds.formLayout(viewLayout,e=True, attachForm= (self.cb_base_meshes, 'bottom', 2) )

        cmds.formLayout(viewLayout,e=True, attachControl=(self.cb_curves,'left', 2, self.cb_base_meshes))
        cmds.formLayout(viewLayout,e=True, attachForm= (self.cb_curves, 'top', 2) )
        cmds.formLayout(viewLayout,e=True, attachForm= (self.cb_curves, 'bottom', 2) )

        cmds.formLayout(viewLayout,e=True, attachControl=(self.cb_result_mesh,'left', 2, self.cb_curves))
        cmds.formLayout(viewLayout,e=True, attachForm= (self.cb_result_mesh, 'top', 2) )
        cmds.formLayout(viewLayout,e=True, attachForm= (self.cb_result_mesh, 'bottom', 2) )

        cmds.formLayout(viewLayout,e=True, attachControl=(self.btn_lock_unlock_res,'left', 2, self.cb_result_mesh))
        cmds.formLayout(viewLayout,e=True, attachForm= (self.btn_lock_unlock_res, 'top', 2) )
        cmds.formLayout(viewLayout,e=True, attachForm= (self.btn_lock_unlock_res, 'bottom', 2) )

        # ---------------------------------------- MAIN LAYOUT ORGANIZATION ------------------------------------------ #
        cmds.formLayout(layout,e=True, attachForm=(globalsLayout,'top', 10))
        cmds.formLayout(layout,e=True, attachForm=(globalsLayout,'left', 2))
        cmds.formLayout(layout,e=True, attachForm=(globalsLayout,'right', 2))

        cmds.formLayout(layout,e=True, attachControl=(operatorsLayout,'top', 10, globalsLayout))
        cmds.formLayout(layout,e=True, attachControl=(operatorsLayout,'bottom', 10, viewLayout))
        cmds.formLayout(layout,e=True, attachForm=(operatorsLayout,'left', 2))
        cmds.formLayout(layout,e=True, attachForm=(operatorsLayout,'right', 2))

        cmds.formLayout(layout,e=True, attachForm=(viewLayout,'bottom', 2))
        cmds.formLayout(layout,e=True, attachForm=(viewLayout,'left', 2))
        cmds.formLayout(layout,e=True, attachForm=(viewLayout,'right', 2))

        cmds.setParent( '..' )
        cmds.setParent( '..' )
        cmds.setParent( '..' )

    # ------------------------------------------------ TREE CALLBACKS ------------------------------------------------ #
    # def __item_double_click(self, *args):
    # 	print args
    def __get_item_type(self, item_name):
        """Return the item type"""

        # If the item is not a leaf item
        if not cmds.treeView( self.tree_view_control, q=True, itemParent = item_name):
            return self.ITEM_TYPE_NETWORK

        elif cmds.treeView( self.tree_view_control, q=True, isLeaf = item_name):

            if item_name.startswith('A'):
                return self.ITEM_TYPE_INPUT_A

            elif item_name.startswith('B'):
                return self.ITEM_TYPE_INPUT_B

            elif item_name.startswith(self.__item_result_name):
                return self.ITEM_TYPE_RESULT
        else:
             return self.ITEM_TYPE_OTHER

    def __edit_label_rename(self, *args):
        """Rename the operator"""
        item_name = args[0]

        if self.__get_item_type(item_name) == self.ITEM_TYPE_NETWORK:
            item_new_name = self.__validate_new_name( args[1] )
            item_index = cmds.treeView( self.tree_view_control, q=True, itemIndex = item_name)

            operators = self.handler_instance.get_operations_list()
            operators[item_index].get_network_object().set_name(item_new_name)

            cmds.select(cmds.ls(sl=True), r=True)

    def __selectTreeCallBack(self, *args):
        """
        When changing the tree element selection:
            Change the attributes of the active node
            Update the visibility section
            Change the in meshes and curves color
        """
        # Get the callback values
        item_name = cmds.treeView( self.tree_view_control, q=True, selectItem=True)[0]

        item_type = self.__get_item_type(item_name)
        selected = cmds.ls(sl=True)[0]

        if  item_type == self.ITEM_TYPE_NETWORK:
            item_index = cmds.treeView( self.tree_view_control, q=True, itemIndex = item_name)

            # Set the current selection as the current operator in the handler
            self.handler_instance.set_current_operator( item_index )

            # Activate the color feedback feature
            self.handler_instance.set_in_mesh_color_hilighting(1)
            self.handler_instance.set_curves_color_hilighting(1)

            # Update visibility of objects if in local mode
            if self.handler_instance.local_visibility:
                self.toggle_local_global_visibility()

            # Send out that we need to update the attributes
            self.__update_attributes()


            # print item_name, 'index is:', item_index
            if self.VERBOSE: print 'selection :- str= ' + args[0] + ' onoff= ' + str(args[1])

        elif item_type in (self.ITEM_TYPE_INPUT_A, self.ITEM_TYPE_INPUT_B):
            input_parent = cmds.treeView(self.tree_view_control, q=True, itemParent=item_name)
            net_parent = cmds.treeView(self.tree_view_control, q=True, itemParent=input_parent)

            parent_index = cmds.treeView(self.tree_view_control, q=True, itemIndex=net_parent)
            item_index = cmds.treeView(self.tree_view_control, q=True, itemIndex=item_name)

            associated_op = self.handler_instance.get_operations_list()[parent_index]

            in_mesh = associated_op.get_input_meshes()[item_index]
            cmds.select(in_mesh, r=True)

        elif item_type == self.ITEM_TYPE_RESULT:
            parent = cmds.treeView(self.tree_view_control, q=True, itemParent=item_name)

            item_index = cmds.treeView(self.tree_view_control, q=True, itemIndex=parent)

            associated_op = self.handler_instance.get_operations_list()[item_index]

            res_mesh = associated_op.get_result_mesh()

            if selected != res_mesh:
                cmds.select(res_mesh, r=True)

        # Change the selection of the tree view by returning true...
        return True

    def __change_visibility_TreeCallBack(self, *args):
        """
        Show the result mesh for the specified pressed operator
        """
        # Get the callback values
        item_name = args[0]
        item_index = cmds.treeView( self.tree_view_control, q=True, itemIndex = item_name)

        # Show only the specified result mesh
        self.handler_instance.show_operator_result_for(item_index)

        # Update the icon
        self.set_visibility_icon(item_name)


        if self.VERBOSE: print 'press :- str= ' + args[0] + 'FIRST onoff= ' + str(args[1])

    def __bypass_operator_TreeCallBack(self, *args):
        """
        Set the second icon image when it is clicked
        """
        # Get the button data
        item_name = args[0]
        item_status = int(args[1])

        item_index = cmds.treeView( self.tree_view_control, q=True, itemIndex = item_name)

        # Set network status
        net_name = item_name.split(' ')[0]
        cmds.setAttr( net_name + '.' + nc.operation_status_name, item_status )

        # Set the operator status
        self.handler_instance.bypass_operator(item_index, item_status)

        # Update the icon
        self.set_bypass_icon(item_name, item_status)


        if self.VERBOSE: print 'press :- str= ' + args[0] + 'SECOND onoff= ' + str(args[1])

    def __dragCallback(self, *args):
        """
        Pass the new order of operations to the handler
        """
        # Id of positions
        #
        curr_id = args[2][0]
        new_id = args[4][0]

        # These two rapresents the networks that will come before and after
        # the current new position of the moved operator
        next_network = args[5]
        prev_network = args[6]

        if self.VERBOSE: print 'Moving current operator %s from %s to %s' % (args[0], curr_id, new_id)
        if self.VERBOSE: print 'We will connect its out to %s and its in to %s' % (next_network, prev_network)

        # Reorder operators
        #
        update_stack = self.handler_instance.move_operator(curr_id, new_id)

        if update_stack:
            if self.VERBOSE: print 'Valid operation, refreshing'
            self.__update_tree_view()
        else:
            curr = cmds.ls(sl=True)
            cmds.select(cl=True)
            cmds.select(curr)
            if self.VERBOSE: print 'Invalid operation, refreshing aborted'

    def __store_tree_state(self, *args):
        """Store the state"""
        item_name = args[0]
        is_expanded = cmds.treeView(self.tree_view_control, q=True, isItemExpanded=item_name)
        self.items_state[item_name] = not is_expanded

    # ------------------------------------------------- UI UPDATING -------------------------------------------------- #
    def update(self):
        """
        When this method is called all the UI elements of the Operator stack
        are updated according to the internal HmNetworkHandler class
        """
        if self.UPDATE_ON_CLICK:
            if self.handler_instance.get_operations_list():
                self.__update_tree_view()
                self.__update_compute()
                self.__update_visibility()
                self.__update_attributes()

        # Activate the color feedback feature
        self.handler_instance.set_in_mesh_color_hilighting(1)
        self.handler_instance.set_curves_color_hilighting(1)

    def activate(self):
        """Activate the UI elements"""
        if self.UPDATE_ON_CLICK:
            cmds.frameLayout( self.main_parent, edit = True, enable = True )

    def deactivate(self):
        """Deactivate the UI elements"""

        if self.UPDATE_ON_CLICK:
            # If we deactivate the control we always want to clean the list
            self.__clean_tree_view()
            cmds.frameLayout( self.main_parent, edit = True, enable = False )

            try:
                # Deactivate the color feedback feature
                self.handler_instance.set_in_mesh_color_hilighting(0)
                self.handler_instance.set_curves_color_hilighting(0)
            except:
                cmds.warning('HM: Nothing to deactivate from color Hilighting')

            # Clean the memory
            self.handler_instance.reset()


    # ---------------------------------------------- INTERNAL METHODS ------------------------------------------------ #
    def __validate_new_name(self, name):
        """ Validate the new name by making sure that no label is passed"""
        # Rapresent if a dirty name is found
        clean_name = False
        replace_string = ''
        out_name = name

        # Check if one of the labels is passed in the name
        for label in self.__OPERATION_LABELS:

            # Compose the string with the current separator
            current_label = label + ' - '
            # print 'comparo', current_label, name

            # Check if we recieve it in the name
            if current_label in name:
                # print 'trovato! cazzoooo'
                clean_name = True
                replace_string = current_label
                break
            else:
                clean_name = False
                # print 'nulla'

        # Rename if we have found the string
        if clean_name:
            # print replace_string, name
            out_name = name.replace(replace_string, '')
            # print 'il nome nuovo', out_name

        return out_name

    def __update_compute(self):
        """ When this function is called the compute check box is updated based on the state of current network"""
        # Get the data from the handler
        com_val = self.handler_instance.get_compute_state()
        # Set to the UI
        cmds.checkBox(self.cb_global_compute, e= True, v = com_val )

    def __update_visibility(self):
        """When this function is called the visibility part is updated based on the state of current network"""
        # Get the data from the handler
        in_shading, base_vis, curves_vis, result_vis, result_lock, local_global = self.handler_instance.get_objects_visibility()

        # Set the global or local radio button
        # cmds.radioCollection(self.global_local_collection, e=True, select= self.__LOC_GLOB_NAMES[local_global] )
        cmds.radioButtonGrp(self.global_local_coll, e=True, select=local_global + 1)

        # Set the visibility on the UI
        cmds.checkBox(self.cb_base_meshes, e= True, v = base_vis )
        cmds.checkBox(self.cb_curves, e= True, v = curves_vis )
        cmds.checkBox(self.cb_result_mesh, e= True, v = result_vis )

        # Set the right icon for the lock
        self.__set_lock_icon(result_lock)
        self.__set_wire_shading_icon(in_shading)

    def __update_attributes(self):
        """This function send out a value when we need to update the attributes editing UI"""
        # So we update an external object from here... missing qt signals right now...
        if self.attribute_object:
            compound = self.handler_instance.get_current_operator().get_compound_node().get_name()
            offset = self.handler_instance.get_current_operator().get_offset_node().get_name()
            in_meshes = self.handler_instance.get_current_operator().get_input_meshes()
            self.attribute_object.connectCompoundNode(compound, offset, in_meshes )


    # ========== Mini Icons update
    def __set_lock_icon(self, on_off):
        """      Set the result icon on or off
        based on the 1 or zero value
        """
        icon_image = "Lock_ON.png" if on_off else "Lock_OFF.png"
        cmds.shelfButton(self.btn_lock_unlock_res, edit = True, image1= icon_image)

    def __set_wire_shading_icon(self, on_off):
        """      Set the result icon on or off
        based on the 1 or zero value
        """
        icon_image = "Shaded.png" if on_off else "WireFrame.png"
        cmds.shelfButton(self.btn_wire_shad_base, edit = True, image1= icon_image)

    def __show_hide_in_meshes_and_update_lock_icon(self, *args):
        """      We create this function for updating automatically the lock
        icon in case the option for the automatic lock unlock is on
        """
        value = args[0]

        self.handler_instance.set_in_mesh_visibility(value)

        # print 'self.AUTO_LOCK_UNLOCK_RESULT =', self.AUTO_LOCK_UNLOCK_RESULT
        if self.AUTO_LOCK_UNLOCK_RESULT:
            self.__set_lock_icon(value)
            self.handler_instance.set_out_mesh_lock(value)

    def set_bypass_icon(self, item_name, on_off):
        """      Write here what the function does.
        
        Args:
            item_name: The name of the current tree view element
            on_off: the boolean value for turning the icon on or off
        
        Returns:
            None

        """
        state = ['buttonUp', 'buttonDown']

        # Chose icon type
        if item_name == self.items_name_list[0]:
            # End item
            status_icon = self.__icon_status_end[on_off]
        else:
            # Normal Item
            status_icon = self.__icon_status_mid[on_off]


        # If we are not on the first operator, change the button
        if not item_name == self.items_name_list[-1]:
            # Set the new icon and force button state
            cmds.treeView( 	self.tree_view_control, e=True,
                            image = [item_name, 3,  status_icon],
                            buttonState = [item_name, 3, state[on_off] ] )

    def set_visibility_icon(self, item_name):
        """      Set the current input item icon ON and all the others OFF
        """      # Set the icon on for the current and off for all the others
        for item in self.items_name_list:
            if item == item_name:
                cmds.treeView( self.tree_view_control, e=True, image = [item, 1,  self.__visible_icons[1]],  )
            else:
                cmds.treeView( self.tree_view_control, e=True, image = [item, 1,  self.__visible_icons[0]],  )


    # ============== TREE VIEW INTERNAL METHODS ==============
    def __update_tree_view(self):
        """      When this function is called the tree view is updated with the current handler informations
        
        Args:
            param1: This is the first param.
            param2: This is a second param.
        
        Returns:
            None
        """
        self.__clean_tree_view()
        self.sub_item_selected = False

        # Get the UI updates from the Handler
        current_operations = self.handler_instance.get_operations_list()
        operations_num = len(current_operations) -1

        # Populate the tree view
        for x, operation in enumerate(current_operations):

            # Determinate the secondary icon type
            if x == 0:
                sec_icon_type = 2
            elif x == operations_num:
                sec_icon_type = 0
            else:
                sec_icon_type = 1

            # Add the item to the tree view
            item_name = self.__add_operator_element( operation, x, sec_icon_type )

            # Add the item to the global list
            self.items_name_list.append(item_name)

        # Hilight the item containing the object that has initialized hmHandler if nothing already selected
        if not self.sub_item_selected:
            selected_name = self.items_name_list[self.handler_instance.get_current_operator_ID()]
            cmds.treeView( self.tree_view_control, e=True, selectItem = [selected_name, 1])


    def __clean_tree_view(self):
        """      This method clean every object in the tree view
        
        Args:
            None
        
        Returns:
            None
        """
        cmds.treeView( self.tree_view_control, edit=True, removeAll = True )
        self.items_name_list = []


    def __add_operator_element(self, operator_object, incremental, secondary_icon_type ):
        """      Add an operator element to the tree view
        
        Args:
            operator_object: Add here the HmOperatorObject to add in the stack.
            incremental: for avoiding name clashes in the tree view we need an incremental number when creating N elements
            icon_type: Set the secondary icon type:
                                                    0 = Start of chain
                                                    1 = Mid of the chain
                                                    2 = End of the chain
        
        Returns:
            String of the layer item name created
        """
        spacing = ' ' * incremental

        def basename(name):
            "remove the dagpath"
            if '|' in name:
                return name.split('|')[-1]
            return name

        def was_item_expanded(item_name):
            if item_name in self.items_state:
                return self.items_state[item_name]
            return False

        # === ADD THE MAIN ROOT OPERATION LAYER

        # Basic data
        selected = cmds.ls(sl=True)[0]
        compound = operator_object.get_compound_node()
        net_name = operator_object.get_network_name()
        oper_ID = compound.get_attribute(nc.compound_operation)
        input_meshes = operator_object.get_input_meshes()
        result_mesh = operator_object.get_result_mesh()

        # Eye icon
        # Determinate if the current operator result is Visible
        is_visible = result_mesh.get_attribute('visibility')
        eye_icon = self.__visible_icons[is_visible]

        # Operator icon
        network_item_name = net_name[1:] + ' - ' + self.__OPERATION_LABELS[ oper_ID ]
        oper_icon = self.__OPERATION_ICONS[ oper_ID ]

        # Bypass Icon
        net_status = 0 if cmds.getAttr( net_name + '.' + nc.operation_status_name ) else 1 # we invert the value, cause of the choice
        state = ['buttonUp', 'buttonDown']


        # Create a different icon for the ending part
        if secondary_icon_type == 0 :
            stat_icon =self.__icon_status_start[net_status]
        elif secondary_icon_type == 1:
            stat_icon =self.__icon_status_mid[net_status]
        elif secondary_icon_type == 2:
            stat_icon =self.__icon_status_end[net_status]

        # Create the main item
        cmds.treeView( self.tree_view_control, e=True, addItem = (network_item_name, '') ) # To implement: itemRenamedCommand,

        # Set the main icon and beavhior
        cmds.treeView( self.tree_view_control, e=True,
                        image = [network_item_name, 1, eye_icon],
                         )

        # Set the second icon and beavhior
        cmds.treeView( self.tree_view_control, e=True,
                        image = [network_item_name, 2, oper_icon],
                        ignoreButtonClick = [network_item_name, 2, 1],
                         )
        # Status image
        cmds.treeView( self.tree_view_control, e=True,
                        image = [network_item_name, 3, stat_icon ],
                        buttonStyle = [network_item_name, 3, '2StateButton'],
                        buttonState = [network_item_name, 3, state[net_status] ],
                        ignoreButtonClick = [network_item_name, 3, 1 if secondary_icon_type == 0 else 0],
                         )

        # Expanded state
        cmds.treeView( self.tree_view_control,
                       e=True,
                       expandItem = [network_item_name, was_item_expanded(network_item_name)] )



        # Add the input meshes and the intersections as children of the tree view element
        selected_in_mesh_label = None

        if self.ITEMS_INCLUDES_HIERARCHY:
            # === ADD THE INPUT MESHES
            input_meshes_label = self.__item_mesh_group_name + spacing

            # Mesh Group
            cmds.treeView(self.tree_view_control,
                          e=True,
                          addItem = (input_meshes_label, network_item_name),
                          allowReparenting=False,
                          hideButtons=True,)

            # Mesh Items
            for x, mesh in enumerate(input_meshes):
                # Create two different labels for the A and B side
                #
                mesh_fullname = mesh.get_name()
                mesh_name = basename(mesh_fullname)

                if x == 0:
                    current_label = self.__item_mesh_A_prefix + mesh_name
                else:
                    current_label = self.__item_mesh_B_prefix[:1] + str(x) + self.__item_mesh_B_prefix[1:]  + mesh_name

                if selected in mesh_fullname:
                    selected_in_mesh_label = current_label

                cmds.treeView(self.tree_view_control,
                              e=True,
                              addItem = (current_label, input_meshes_label),
                              allowReparenting=False,
                              hideButtons=True )

            # # Expanded state
            cmds.treeView( 	self.tree_view_control,
                            e=True,
                            expandItem = [input_meshes_label, was_item_expanded(input_meshes_label)],
                            buttonVisible = [(input_meshes_label, 2, False), (input_meshes_label, 3, False) ],
                            image = [input_meshes_label, 1, eye_icon] )


            # # === ADD THE INTERSECTION CURVES
            #
            # intersection_curves = operator_object.get_intersection_curves()
            # intersection_curves_label = self.__item_intersection_group_name + spacing
            #
            # # Intersecions Group
            # cmds.treeView(
            #     self.tree_view_control,
            #     e=True,
            #     addItem = (intersection_curves_label, network_item_name),
            #     allowReparenting=False)#hideButtons=True  )
            #
            #
            # # Intersecions Items
            # for curve in intersection_curves:
            #     current_label = curve.get_transform().get_name().split('|')[-1] + spacing
            #     cmds.treeView( self.tree_view_control,
            #                    e=True,
            #                    addItem = (current_label, intersection_curves_label),
            #                    allowReparenting=False,
            #                    hideButtons=True )
            #
            # # Collapse, hide 2 buttons and set the visibility icon
            # cmds.treeView( self.tree_view_control, e=True, expandItem = [intersection_curves_label, False],
            #                 buttonVisible = [(intersection_curves_label, 2, False), (intersection_curves_label, 3, False) ],
            #                 image = [intersection_curves_label, 1, eye_icon] )

            # === ADD THE RESULT MESH


            result_name = basename(result_mesh.get_name())
            result_mesh_label = self.__item_result_name + result_name

            # Result mesh
            cmds.treeView( self.tree_view_control,
                           e=True,
                           addItem = (result_mesh_label, network_item_name),
                           allowReparenting=False,
                           hideButtons=True  )


            # # Expanded state
            cmds.treeView( self.tree_view_control,
                           e=True,
                           expandItem = [result_mesh_label, was_item_expanded(result_mesh_label)],
                           buttonVisible = [(result_mesh_label, 2, False), (result_mesh_label, 3, False) ],
                           image = [result_mesh_label, 1, eye_icon] )

        # selected part
        if self.ITEMS_INCLUDES_HIERARCHY and not self.sub_item_selected:
            # print input_meshes, selected

            if selected == result_name:
                cmds.treeView(self.tree_view_control, e=True, expandItem=[network_item_name, True])
                cmds.treeView(self.tree_view_control, e=True, selectItem=[result_mesh_label, 1])
                self.sub_item_selected = True
            elif selected_in_mesh_label:
                cmds.treeView(self.tree_view_control, e=True, expandItem=[network_item_name, True])
                cmds.treeView(self.tree_view_control, e=True, expandItem=[input_meshes_label, True])
                cmds.treeView(self.tree_view_control, e=True, selectItem=[selected_in_mesh_label, 1])
                self.sub_item_selected = True



        # === Return the item name
        return network_item_name


    #================================================
    #==================  COMMANDS ===================
    #================================================
    def toggle_local_global_visibility(self, *args):


        # Get the UI values
        # local_val = True if cmds.radioCollection(self.global_local_collection, q=True, select= True ) == self.__LOC_GLOB_NAMES[1] else False
        local_val = cmds.radioButtonGrp(self.global_local_coll, q=True, select= True) - 1
        in_mesh_vis = cmds.checkBox(self.cb_base_meshes, q= True, v = True )
        curve_vis = cmds.checkBox(self.cb_curves, q= True, v = True )


        if self.VERBOSE: print 'Local visibility is: ', local_val, '\nin meshes:', in_mesh_vis, '\ncurves: ', curve_vis

        # Set to the handler
        self.handler_instance.set_local_visibility(local_val, in_mesh_vis, curve_vis)


    def change_smooth(self, smooth_direction):
        """      Increase the smooth and update the UI

        Args:
            smooth_direction: 1 means increase, 0 means decrease

        """
        self.handler_instance.set_new_net_smooth(smooth_direction)
        self.__update_attributes()


    def toggle_wire_shade_input(self, *args):
        """
        Lock unlock the result meshes and ghange icon accordingly
        """
        current = self.handler_instance.get_objects_visibility()[0]

        # Get the opposite of the current
        toggle_value = 0 if current else 1

        # Set to handler and change icon
        self.handler_instance.set_in_mesh_shading(toggle_value)
        self.__set_wire_shading_icon(toggle_value)


    def toggle_lock_result(self, *args):
        """
        Lock unlock the result meshes and ghange icon accordingly
        """
        current = self.handler_instance.get_objects_visibility()[4]

        # Get the opposite of the current
        toggle_value = 0 if current else 1

        # Set to handler and change icon
        self.handler_instance.set_out_mesh_lock(toggle_value)
        self.__set_lock_icon(toggle_value)

    def wireframeToggle(self, *args):
        """
        Turns on and off the selection hilighting for the current persp view
        """
        perspPanel = cmds.getPanel( withFocus=True)

        if (cmds.modelEditor( perspPanel, query=1, wireframeOnShaded=1)) == 1:
            cmds.modelEditor( perspPanel, edit=True, wireframeOnShaded=0)
        else:
            cmds.modelEditor( perspPanel, edit=True, wireframeOnShaded=1)

    def hilightingToggle(self, *args):
        """
        Turns on and off the selection hilighting for the current persp view
        """
        # perspPanel = cmds.getPanel( withLabel='Persp View')
        perspPanel = cmds.getPanel( withFocus=True)

        if (cmds.modelEditor( perspPanel, query=1, sel=1)) == 1:
            cmds.modelEditor( perspPanel, edit=True, sel=0)
        else:
            cmds.modelEditor( perspPanel, edit=True, sel=1)


    #================================================
    #==================  TEMPORARY ===================
    #================================================
    def update_isolated_operator(self, active):
        """
        Gray out the operators not currently isolated...

        Args:
            active: True or False
        """
        def activate_deactivate_items(value, exclude = None):
            """
            exclude: item name to exclude from the operation
            value: 1 for activating, 0 for deactivating
            """

            # Create a copy of the list
            item_list = list(self.items_name_list)

            # Exclude the element from the operation
            if exclude:
                item_list.remove(exclude)

            for item in item_list:
                # Set the item
                cmds.treeView( 	self.tree_view_control, e=True, enableLabel = [item,  value] )
                cmds.treeView( 	self.tree_view_control, e=True, enableButton = [item, 1, value] )
                cmds.treeView( 	self.tree_view_control, e=True, enableButton = [item, 2, value] )
                cmds.treeView( 	self.tree_view_control, e=True, enableButton = [item, 3, value] )


        current_item_name = self.items_name_list[ self.handler_instance.get_current_operator_ID() ]

        if active:
            # print 'lascio acceso soltanto:', current_item_name
            activate_deactivate_items(0, current_item_name)
        else:
            # print 'Accendo tutto!'
            activate_deactivate_items(1, current_item_name)
