#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64
import codecs
import os.path
import os
import sys
import http.client

import json

import ctypes

import struct
import platform


def get_sys():
    '''
    Return the system type with an int
    0 = windows
    1 = mac
    2 = linux
    '''

    _running_OS = platform.system()

    if 'Darwin' in _running_OS:
        # print 'Siamo su un MAC'
        # extension = 'dylib'
        os_type = 1
    else:
        if 'Linux' in _running_OS:
            # print 'Siamo su Linux'
            # extension = 'so'
            os_type = 2
        else:
            # print 'Siamo su Windows'
            # extension = 'dll'
            os_type = 0

    # result_name = 'hmUtility' + str(struct.calcsize("P")*8) + '.' + extension
    # print result_name

    return os_type


def get_mac(path):
    # Define a list with the extensions for every os
    ext_list = ['dll', 'dylib', 'so']

    # Create the name of the utility with the right extension
    hmUtility = 'hmUtility' + str(struct.calcsize("P") * 8) + '.' + ext_list[get_sys()]

    # Build the path
    pathToLib = os.path.join(path, hmUtility)
    print('path to lib', pathToLib)
    # print 'usero', pathToLib

    # Do c++ calling stuff
    lib = ctypes.CDLL(os.path.normpath(pathToLib))
    lib.free_mac.argtypes = ctypes.c_void_p,
    lib.free_mac.restype = None
    lib.get_mac.argtypes = []
    lib.get_mac.restype = ctypes.c_void_p

    ptr = lib.get_mac()

    my_mac = ctypes.cast(ptr, ctypes.c_char_p).value
    # print cast(ptr, c_char_p).value
    # print ' My Mac = ' + my_mac
    lib.free_mac(ptr)

    return my_mac


class LicFile:
    def __init__(self, path, mac):
        self.path = path
        self.mac = mac
        self.filename = os.path.join(path, 'hmTools.lic')

    @property
    def state(self):
        return os.path.isfile(self.filename)

    def setfileName(self, name):
        self.filename = name

    def getfileName(self):
        return self.filename

    def createLicFile(self, key):
        if not self.state:
            with codecs.open(self.filename, 'w', 'utf-8') as f:
                text = 'Good for you to find the solution but please leave us alone ' \
                       'we dont even have the money to pay for a proper licensing ' \
                       'system please keep it for yourself thanks'
                text_out = base64.b64encode(bytes('{};{};{}'.format(text, self.mac, key), encoding='utf-8'))
                f.write(text_out.decode(encoding='utf-8'))

    def deleteLicFile(self):
        if self.state:
            os.remove(self.filename)

    def updateLicFile(self, key):
        if not self.state:
            self.createLicFile(key)
        else:
            with codecs.open(self.filename, 'a', 'utf-8') as f:
                f.write(str(key) + ";")

    def getLicFile(self):
        if not self.state:
            return None
        else:
            with codecs.open(self.filename, 'r', 'utf-8') as f:
                data = f.read()
                data = base64.b64decode(data)
                data_s = str(data).split(';')
                out_data = json.loads(data_s[-1])
                return out_data


class MyLic:
    DEACTIVATE_SUCCESSFUL = 0
    DEACTIVATE_LIMIT_REACHED = 1
    DEACTIVATE_ON_INVALID_LICENSE = 2
    DEACTIVATE_ERROR = 3
    UNABLE_TO_CONNECT = 4

    def __init__(self, path, mac):
        print('INITIALIZED PY3 LIC CLASS')
        self.mac = mac
        self.path = path
        self.url = 'www.pux-web.com'
        self.lic_file = LicFile(path, mac)

    @property
    def state(self):
        return self.lic_file.state

    def getLicense(self):
        return self.lic_file.getLicFile()

    def sendPost(self, post_url, service_url, mac, key, data_service):
        headers = {"Content-type": "application/json",
                   "Accept": "text/plain"}
        data = data_service
        params = json.dumps(data, separators=(',', ':'))

        try:
            conn = http.client.HTTPConnection(post_url)
            conn.request('POST', f'http://{post_url}{service_url}', params, headers)
            response = conn.getresponse()
            demiurgo = response.read()
            status = response.status
            message = response.msg
            return True, demiurgo, status, message
        except:
            return False, 'Missing connection with Server PUX', 400, ''

    def activateLic(self, key_str, software='maya'):
        if software == 'maya':
            post_url = '/auth/hardmesh'
        elif software == 'max':
            post_url = '/auth/hardmesh3ds'
        else:
            raise ValueError(f'Software {software} not recognized. Please choose "maya" or "max"')

        data = {"cavernasecretkey": "fattilicazzitui", 'mac': str(self.mac), 'key': key_str}
        result, response, status, message = self.sendPost(self.url, post_url, self.mac, key_str, data)

        response = response.decode(encoding='utf-8')
        print('response', response)

        if result:

            if 'falcomataaaaaaa' in response:
                return False, ' License key not valid.'
            else:
                if 'tuo Osho' in response:
                    self.lic_file.deleteLicFile()
                    self.lic_file.createLicFile(response)
                    return True, 'License succesfully activated!'
        else:
            return False, 'No Connection to PUX Server, unable to proceed.' \
                          '\nPlease make sure that no Firewall or Anti Virus are blocking the outgoing connections.'